@@ -1,7 +1,7 @@
 #
 # Run this script to generated a faq.html output file
 #
-set rcsid {$Id: faq.tcl,v 1.40 2007/09/04 01:58:27 drh Exp $}
+set rcsid {$Id: faq.tcl,v 1.37 2007/06/09 09:53:51 drh Exp $}
 source common.tcl
 header {SQLite Frequently Asked Questions</title>}
 
@@ -154,8 +154,8 @@ faq {
   <p>When SQLite tries to access a file that is locked by another
   process, the default behavior is to return SQLITE_BUSY.  You can
   adjust this behavior from C code using the 
-  <a href="capi3ref.html#sqlite3_busy_handler">sqlite3_busy_handler()</a> or
-  <a href="capi3ref.html#sqlite3_busy_timeout">sqlite3_busy_timeout()</a>
+  <a href="capi3ref#sqlite3_busy_handler">sqlite3_busy_handler()</a> or
+  <a href="capi3ref#sqlite3_busy_timeout">sqlite3_busy_timeout()</a>
   API functions.</p>
 }
 
@@ -163,18 +163,21 @@ faq {
   Is SQLite threadsafe?
 } {
   <p>Yes.  Sometimes.  In order to be thread-safe, SQLite must be compiled
-  with the SQLITE_THREADSAFE preprocessor macro set to 1.  Both the windows
-  and linux precompiled binaries in the distribution are compiled this way.
-  If you are unsure if the SQLite library you are linking against is compiled
-  to be threadsafe you can call the
-  <a href="capi3ref.html#sqlite3_threadsafe">sqlite3_threadsafe()</a>
-  interface to find out.
-  </p>
+  with the THREADSAFE preprocessor macro set to 1.  In the default
+  distribution, the windows binaries are compiled to be threadsafe but
+  the linux binaries are not.  If you want to change this, you'll have to
+  recompile.</p>
+
+  <p>"Threadsafe" in the previous paragraph means that two or more threads
+  can run SQLite at the same time on different "<b>sqlite3</b>" structures
+  returned from separate calls to 
+  <a href="capi3ref#sqlite3_open">sqlite3_open()</a>.  It is never safe
+  to use the same <b>sqlite3</b> structure pointer in two
+  or more threads.</p>
 
   <p>Prior to version 3.3.1,
   an <b>sqlite3</b> structure could only be used in the same thread
-  that called <a href="capi3ref.html#sqlite3_open">sqlite3_open</a>
-  to create it.
+  that called <a href="capi3ref#sqlite3_open">sqlite3_open</a> to create it.
   You could not open a
   database in one thread then pass the handle off to another thread for
   it to use.  This was due to limitations (bugs?) in many common threading
